# mapspacer_sketchup
MapSpacer™ SketchUp Extension (written in Ruby) is installed as a plugin for the SketchUp design software.

The following documentation is available for new developers at concept3D to reference to understand how MapSpacer™ works, and for reviewers at Trimble (SketchUp's parent company) to use when "reviewing" the extension(s) before it can be distributed on SketchUp's Extension Warehouse.

## Screenshot
![MapSpacer screenshot](/c3d_mapspacer/images/MapSpacer-screenshot.png?raw=true "MapSpacer screenshot")

## Purpose
MapSpacer™ is a powerful space planning SketchUp Extension that lets clients envision their entire space down to the last square inch. They can view in 2D and 3D, make changes, reset and do it again! 

The user will open a SketchUp model file of they space they would like to rent/purchase. A summary of the count of each component added will be updated as the space configuration changes. Screenshots can be saved of each space. This summary, the screenshots, and the modified model itself can then be uploaded to the Request For Proposal (RFP) portion of the venue's website. This RFP is the basis to reserve a space for a specific date and have the space configured how the user wants it.

## Extension Audience
Users of this extension come in two flavors:
1. The Event Planner (EP) is the "owner or sales person" of the venue offering the space. The EP will use MapSpacer™ to create a space for one of their clients, or modify a submitted design.
1. The Meeting Planner (MP) is the person who is looking for a space to rent/purchase. They will download a model file of a potential space, and the MP will make that space look like what they want for their event.

## Summary
When a MapSpacer™ model is opened, a new window will appear that will help users add components to the space they want to modify. If the room has breakout spaces (i.e. subrooms that can be turned on/off), the user can select them in the Breakout Spaces section. Using the Save Snapshot and Export Room Report buttons, the user will create a folder of files that can be uploaded to the RFP site that is opened using the "Request a Proposal" button at the bottom.

## MapSpacer™ Model Detection
If there is an Attribute Dictionary with the name "c3d_mapspacer", then the model will be considered a validMapSpacerFile. However, we have several models in the public that used a different identification process that we want to consider valid also. For those, in the Window -> Model Info menu option, if a model has a specific string in the Name attribute, it will be considered a validMapSpacerFile. If validMapSpacerFile, then the MapSpacer™ main window will open. If not, then the user can continue to work on the model as if MapSpacer™ is not installed.

## Multiple Install Requirement
The core extension is c3d_mapspacer. Each venue (i.e. each convention center, real estate client, etc) will also have a unique extension (prefixed with "c3d_mapspacer_"). This second install step is required to get all the branding (i.e. css and images) and the models (i.e. components) that are unique to that venue. Each venue also has unique "rules" that their spaces follow (i.e. you can't put a chair with X feet of a fire exit). This second extension also contains those rules that help the user create their space.

Depending on the venue detected in the model detection phase (`Model.new()`), MapSpacer™ will dynamically "Sketchup.require" the appropriate file from an installed venue extension (i.e. during initialize() ).

If the attribute dictionary has a venue of just "MapSpacer", then a 'standard' venue called 'mapspacer' will be used, and this standard venue is included in the core extension and, thus, does not need a second installation.

More info about the Venue Support Extensions can be found [here](#venue-support-extensions)

## Architecture
The rest of the extensions architecture can be summarized as:
1. css/, fonts/, html/, js/ are basic directories for their respective content
1. The helpers/ directory contains some helper ruby files
1. The observers/ directory contains all the observer definitions
1. The ui/ directory contians the ruby files used to create the window in the application
1. The venues/ directory contains all the base class definitions, plus a "mapspacer" venue support file. The mapspacer venue contains all the branding and components common to all venues.
1. The models themselves (i.e. the SketchUp files of the rooms) are currently configured (as of MapSpacer™ Version 3.0) to consist of layers that can be turned on/off to change subroom visibility.

## Files of Interest
A few bullet points about some files of interest:
1. main.rb sets up the following instance variables within the C3D::MapSpacer namespace:
* @mainWindow = self::MainWindow.new()
* @menu = self::MapSpacerMenu.new()
* @toolbar = self::MapSpacerToolbar.new()
* @model = self::Model.new()
* @extensionManager = self::ExtensionManager.new()
* @mainObservers = MainMapSpacerObservers.new()

1. js/mapspacer.js contains the JS code that interacts with the html/main.html webpage (i.e. the WebDialog).

1. ui/mainWindow.rb contains the MainWindow class definition. This file will handle all the "communication" between the js/mapspacer.js file and the ruby scripts. The on() function (inherited from window.rb's `class Window < UI::WebDialog`) will accept the commands from javascript (i.e. user interactions in the UI) and MainWindow will use the call() function (inherited in the same file) to communicate from ruby back to javascript (i.e. to update the UI).

1. ui/mainWindow.rb sets up the following instance variables:
* @venueObj = Venue.new()
* @statistics = Statistics.new(@venueObj)

1. The Venue class (i.e. venues/mapspacer/venue.rb which inherits venues/venueBase.rb) (or any of the venue extensions such as c3d_mapspacer_occc/venue.rb which also inherit c3d_mapspacer/venues/venueBase.rb) contain all the "rules" and "custom html" for each venue. The Venue class (specifically the base class) includes an instance variable for LayerRules (layerRules.rb and layerRulesBase.rb). These rules determine when subroom walls should be turned on/off and when we issue a warning to users that the configuration is not valid. The layerDict is a collection of layers from each model that are related (i.e. Airwalls, subrooms, etc). These layers are named with certain rules found in the layerNamingConvention class.

## Venue Support Extensions
Each of the venue support files has a root rb file that describes the venueName and versionNumber, and registers the extension. The main purpose of the extension is to provide support files that are `Sketchup.required()`ed by the core extension. Since all the code to do the registration and set up a command in the Extensions->MapSpacer menu is the same, we simply call a function in the core extension. Therefore, the root rb file, and the loader rb file break from the normal SketchUp logic (in appearance) but ultimate perform the same functions (i.e. registering the extension and modifying the menu). The structure of the support extensions are the same as the venues/mapspacer support files in the core extension.